﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using System.Linq;
using Wisej.Base;
using Wisej.Core;
using Wisej.Core.Design;

namespace Wisej.Web.Ext.TextBoxOpenTip
{
    /// <summary>
    /// Implements the OpenTipValue open source widget in Wisej.
    /// </summary>
    [ToolboxItem(true)]
    [DefaultProperty(nameof(OpenTip))]
    [ProvideProperty("OpenTip", typeof(Control))]
    [Description("OpenTipValue.js widget.")]
    public class OpenTip : Wisej.Web.Component, IExtenderProvider
    {
        // collection of controls with relative tooltip text.
        private Dictionary<Control, string> tooltips;

        #region Constructors

        public OpenTip()
        {
            
        }

        public OpenTip(IContainer container) : this()
		{
            if (container == null)
                throw new ArgumentNullException("container");

            container.Add(this);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Returns or sets whether the ToolTip provider is currently active.
        /// </summary>
        /// <returns>true if the ToolTip is currently active; otherwise, false. The default is true.</returns>
        [DefaultValue(true)]
        [SRDescription("ToolTipActiveDescr")]
        public bool Active
        {
            get
            {
                return this._active;
            }
            set
            {
                if (this._active != value)
                {
                    this._active = value;

                    Update();
                }
            }
        }
        private bool _active = true;


        /// <summary>
        /// Returns or sets the alignment side and position of the tooltips.
        /// </summary>
        [DefaultValue(null)]
        [SRCategory("CatLayout")]
        [SRDescription("ToolTipAlignmentDescr")]
        public Placement? Alignment
        {
            get { return this._alignment; }
            set
            {
                if (this._alignment != value)
                {
                    this._alignment = value;

                    Update();
                }
            }
        }
        private Placement? _alignment = null;


        #endregion

        #region Wisej Implementation


        /// <summary>
        /// Renders the client component.
        /// </summary>
        /// <param name="config">Dynamic configuration object.</param>
        protected override void OnWebRender(dynamic config)
        {
            base.OnWebRender((object)config);
            config.active = this.Active;
            config.alignment = this.Alignment;

            config.className = "wisej.web.ext.TextBoxOpenTip";
            if (this.tooltips != null)
            {
                // send only created controls.
                config.tooltips = this.tooltips
                    .Where(o => o.Key.Created)
                        .Select(o => new
                        {
                            id = ((IWisejControl)o.Key).Id,
                            text = TextUtils.EscapeText(o.Value, true)

                        }).ToArray();

                // register non-created control for delayed registration.
                this.tooltips.Where(o => !o.Key.Created).ToList().ForEach(o =>
                {
                    Control control = o.Key;
                    control.ControlCreated -= this.Control_Created;
                    control.ControlCreated += this.Control_Created;
                });
            }

        }

        #endregion

        #region Methods

        public bool CanExtend(object target)
        {
            return (target is Control);
        }

        /// <summary>
        /// Retrieves the ToolTip text associated with the specified control.
        /// </summary>
        /// <returns>A <see cref="T:System.String" /> containing the ToolTip text for the specified control.</returns>
        /// <param name="control">The <see cref="T:Wisej.Web.Control" /> for which to retrieve the <see cref="T:Wisej.Web.ToolTip" /> text. </param>
        [DefaultValue("")]
        [Localizable(true)]
        [DisplayName("ToolTip")]
        [SRDescription("ToolTipToolTipDescr")]
        [Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
        public string GetToolTip(Control control)
        {
            if (control == null)
                throw new ArgumentNullException("control");

            string text = string.Empty;
            if (this.tooltips != null)
                this.tooltips.TryGetValue(control, out text);

            return text;
        }


        /// <summary>
        /// Removes all ToolTip text currently associated with the ToolTip component.
        /// </summary>
        public void Clear()
        {
            if (this.tooltips != null)
            {
                this.tooltips.ToList().ForEach((o) => {
                    o.Key.Disposed -= this.Control_Disposed;
                    o.Key.ControlCreated -= this.Control_Created;
                });

                this.tooltips.Clear();

                Update();
            }
        }

        /// <summary>
		/// Associates ToolTip text with the specified control.
		/// </summary>
		/// <param name="control">The <see cref="T:Wisej.Web.Control" /> to associate the ToolTip text with. </param>
		/// <param name="caption">The ToolTip text to display when the pointer is on the control. </param>
		public void SetToolTip(Control control, string caption)
        {
            if (control == null)
                throw new ArgumentNullException("control");

            if (String.IsNullOrEmpty(caption))
            {
                if (this.tooltips != null)
                    this.tooltips.Remove(control);
            }
            else
            {
                if (this.tooltips == null)
                    this.tooltips = new Dictionary<Control, string>();

                this.tooltips[control] = caption;

                // remove the control from the extender when it's disposed.
                control.Disposed -= this.Control_Disposed;
                control.Disposed += this.Control_Disposed;
            }

            Update();
        }

        private void Control_Disposed(object sender, EventArgs e)
        {
            Control control = (Control)sender;
            control.Disposed -= this.Control_Disposed;
            control.ControlCreated -= this.Control_Created;

            // remove the extender values associated with the disposed control.
            if (this.tooltips != null)
                this.tooltips.Remove(control);
        }

        private void Control_Created(object sender, EventArgs e)
        {
            // handle the delayed registration of this extender for a control
            // that was not created (not visible) when the extender tried to register it.
            Control control = (Control)sender;
            control.ControlCreated -= this.Control_Created;

            // update the extender, now it will send also this newly created control.
            Update();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                Clear();
            }

            base.Dispose(disposing);
        }

        #endregion
    }
}
